from typing import Dict


class TrustAdapter:
    def __init__(self, use_case: str, context: str, table: str, user_source: str, volume_source: str,
                 rating_source: str, rating_lower: float, rating_upper: float, conversion: float, 
                 certainty: float, cutoff: float, bias_negative: float, bias_positive: float):
        self.use_case = use_case
        self.context = context
        self.table = table
        self.user_source = user_source
        self.volume_source = volume_source
        self.rating_source = rating_source
        self.rating_lower = rating_lower
        self.rating_upper = rating_upper
        self.conversion = conversion
        self.certainty = certainty
        self.cutoff = cutoff
        self.bias_negative = bias_negative
        self.bias_positive = bias_positive

    def to_serializable_dict(self):
        return {
            "use_case": self.use_case,
            "context": self.context,
            "table": self.table,
            "user_source": self.user_source,
            "volume_source": self.volume_source,
            "rating_source": self.rating_source,
            "rating_lower": self.rating_lower,
            "rating_upper": self.rating_upper,
            "conversion": self.conversion,
            "certainty": self.certainty,
            "cutoff": self.cutoff,
            "bias_negative": self.bias_negative,
            "bias_positive": self.bias_positive
        }

    @staticmethod
    def from_serializable_dict(data: Dict):
        if "use_case" not in data.keys() or "context" not in data.keys() or "table" not in data.keys() or "user_source" not in data.keys() or "volume_source" not in data.keys() or "rating_source" not in data.keys() or "conversion" not in data.keys() or "certainty" not in data.keys() or "cutoff" not in data.keys() or "bias_negative" not in data.keys() or "bias_positive" not in data.keys():
            raise ValueError("Missing fields for TrustAdapter!")

        return TrustAdapter(
            data["use_case"],
            data["context"],
            data["table"],
            data["user_source"],
            data["volume_source"],
            data["rating_source"],
            data["rating_lower"],
            data["rating_upper"],
            data["conversion"],
            data["certainty"],
            data["cutoff"],
            data["bias_negative"],
            data["bias_positive"],
        )
