# global imports (dont't worry, red is normal)
import network_constants as netconst
from database.MongoRepositoryBase import MongoRepositoryBase

from lib.database.entities.user import User
from lib.database.entities.user_trust import UserTrust
from lib.database.repositories.user_trust_repository import UserTrustRepository
from typing import List, Dict


class UserRepository(MongoRepositoryBase):
    '''This is a repository for MongoDb.'''

    _user_trust_repository = UserTrustRepository()

    def __init__(self):
        super().__init__(netconst.REPUTATION_CALCULATION_DB_HOSTNAME,
                         netconst.REPUTATION_CALCULATION_DB_PORT,
                         'reputation-computation-db')
        self._collection = 'user'

    def add(self, user: User):
        super().insert_entry(self._collection, user.to_serializable_dict())

    def all(self) -> List[User]:
        result = super().get_entries(
            self._collection, projection={'_id': False})
        result = [User.from_serializable_dict(row) for row in list(result)]

        for user in result:
            self._enrich_user(user)

        return result

    def all_for_use_case(self, use_case: str) -> List[User]:
        result = super().get_entries(self._collection, projection={
            '_id': False}, selection={"use_case": use_case})

        result = [User.from_serializable_dict(row) for row in list(result)]

        for user in result:
            self._enrich_user(user)

        return result

    def one_by_use_case_and_username(self, use_case: str, username: str) -> User:
        result = list(super().get_entries(self._collection, projection={
            '_id': False}, selection={"use_case": use_case, "name": username}))

        if len(result) != 1:
            return None

        user: User = User.from_serializable_dict(result[0])
        self._enrich_user(user)

        return user

    def _enrich_user(self, user: User) -> User:
        user.context_trust = self._user_trust_repository.all_for_user(user)
