from lib.services.trust_service import TrustService
from typing import Dict
import json
import logging

LOGGER = logging.getLogger(__name__)


class MessageHandler:
    def __init__(self, trust_adapter_repository, user_repository, user_trust_repository, trust_trace_repository):
        self.trust_adapter_repository = trust_adapter_repository
        self.user_repository = user_repository
        self.user_trust_repository = user_trust_repository
        self.trust_trace_repository = trust_trace_repository

    def handle(self, body: str):
        # decode the message
        data: Dict = None
        try:
            data = json.loads(body)
        except json.decoder.JSONDecodeError:
            raise ValueError("Invalid Message: Not in JSON format")

        if "type" not in data.keys() or "content" not in data.keys():
            raise ValueError(
                "Invalid Message: Missing fields \"type\" or \"content\"")

        if data["type"] != "new-trace":
            return

        content: Dict = data["content"]

        if "use_case" not in content.keys() or "table" not in content.keys() or "properties" not in content.keys():
            raise ValueError(
                "Invalid Message: Missing fields \"use_case\" or \"table\" or \"properties\"")

        use_case: str = content["use_case"]
        table: str = content["table"]
        properties: Dict = content["properties"]

        trust_service: TrustService = TrustService(
            self.trust_adapter_repository,
            self.user_repository,
            self.user_trust_repository,
            self.trust_trace_repository
        )

        trust_service.updateSystem(use_case, table, properties)