import math
from enum import Enum
from lib.trust.transaction import Transaction
from lib.trust.node import Node
from datetime import datetime
from typing import List, Dict


class TrustLabel(Enum):
    UNTRUSTWORTHY = 1
    UNSURE = 2
    TRUSTWORTHY = 3


class TrustOracle:
    def __init__(self, translate_value: float, knowledge_border: float):
        # translates transaction volume into unitless x-axis value for sigmoid input
        self.translate_value = translate_value
        # nodes must reach a value of at least |trust(node)| = knowledge_border to get a definite label
        self.knowledge_border = knowledge_border

    def rate_node(self, node: Node) -> TrustLabel:
        if node.t < -self.knowledge_border:
            return TrustLabel.UNTRUSTWORTHY

        if node.t > self.knowledge_border:
            return TrustLabel.TRUSTWORTHY

        return TrustLabel.UNSURE

    def calculate_trust_single(self, node_transaction: Transaction):
        return self.calculate_trust([node_transaction])

    def calculate_trust(self, node_transactions: List[Transaction]):
        trust_total = 0

        for transaction in node_transactions:
            transaction_t = abs(transaction.volume / self.translate_value)
            trust_total += transaction_t

        return self.translate_t_to_trust_value(trust_total)

    def translate_t_to_trust_value(self, t_value: float):
        # classic sigmoid function as in https://en.wikipedia.org/wiki/Sigmoid_function
        sig = 1 / (1 + math.exp(-t_value))

        # rescale it st.:
        # values are between ]-1;1[
        sig *= 2
        sig -= 1

        return sig
